#include "interface/ESP32CryptoSource.h"

#include <Ed25519.h>
#include <RNG.h>
#include <NoiseSource.h>

ESP32CryptoSource::ESP32CryptoSource(const char* rngInitTag) {
    RNG.begin(rngInitTag);
    RNG.addNoiseSource(noise);
}

bool ESP32CryptoSource::createPrivateKey(PrivateKey* key) {
    Ed25519::generatePrivateKey(key->bytes);
    return true;
}

bool ESP32CryptoSource::createPublicKey(const PrivateKey* privateKey, PublicKey* publicKey) {
    Ed25519::derivePublicKey(publicKey->bytes, privateKey->bytes);
    return true;
} 

bool ESP32CryptoSource::sign(const uint8_t *message, uint16_t length, Signature* signature, const PrivateKey* privateKey, const PublicKey* publicKey) {
    Ed25519::sign(signature->bytes, privateKey->bytes, publicKey->bytes, message, length);
    return true;
}

bool ESP32CryptoSource::verify(const Signature* signature, const PublicKey* publicKey, const void *message, uint16_t length) {
    return Ed25519::verify(signature->bytes, publicKey->bytes, message, length);
}